/*************************************************************************************************
*
*
*			ObjectMacZapp		-- a standard Mac OOP application template
*
*
*
*			ZApplication.cpp	-- the application object
*
*
*
*
*
*			 1996, Graham Cox
*
*
*
*
*************************************************************************************************/


#include	"ZApplication.h"
#include	"ZEventHandler.h"
#include	"ZWindow.h"
#include	"ZDefines.h"
#include	"ZErrors.h"
#include	<Gestalt.h>

// gApplication is the global application object. There is only one, naturally.

ZApplication*	gApplication = NULL;

// extend these globals to include the file types you can open


short		gNumFileTypes = 2;
OSType		gFileTypeList[2] = {'TEXT', 'PICT'};
OSType		gMacZoopSignature = 'ZAPP';	
Boolean		gIsAColourMac = FALSE;	


static pascal long ZGrowFunc(Size bytesShort);
static GrowZoneUPP	gGZFunc = NewGrowZoneProc(ZGrowFunc);


/*--------------------------------***  CONSTRUCTOR  ***---------------------------------*/

ZApplication::ZApplication()
	: ZCommander(NULL)
{
	long	qdFeatures;
	
	done = FALSE;
	phase = kInitialising;
	zEH = NULL;
	
	shortageFund = NewHandle(kShortageFundSize);
	memIsShort = FALSE;
	userHasSeenAlert = FALSE;
	
	OSErr theErr = Gestalt(gestaltQuickdrawFeatures, &qdFeatures);
	
	if ((theErr == noErr) && (qdFeatures & 1))
		gIsAColourMac = TRUE;
	
	gApplication = this;
}

/*--------------------------------***  DESTRUCTOR  ***---------------------------------*/


ZApplication::~ZApplication()
{
	if (zEH)
		ForgetObject(zEH);
}


/*--------------------------------***  INITMACZAPP  ***---------------------------------*/
/*	

Initialises the application, makes the event handler object and the menubar

----------------------------------------------------------------------------------------*/

void		ZApplication::InitMacZapp()
{
	if (phase == kInitialising)
	{
		InitMacApplication();
		SetGrowZone(gGZFunc);
		MakeEventHandler();
		InitMenuBar();
		
		phase = kRunning;
		
		InitCursor();
	}
}

/*-----------------------------***  INITMACAPPLICATION  ***-----------------------------*/
/*	

Initialises the mac toolbox

----------------------------------------------------------------------------------------*/

	
void		ZApplication::InitMacApplication()
{
	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(NULL);
	
	// clear out the event queue in case any stray clicks or keypresses are left there
	
	FlushEvents(everyEvent,0);

	// give yourself enough memory. For a bigger app, you may need to call MoreMasters a few
	// more times
	
	MaxApplZone();
	MoreMasters();
	
	// see if the program can run on this Mac- if not, we show an alert and exit
	
	if (! CheckCanRun())
	{
		Alert(kCantRunAlertID, NULL);
		ExitToShell();
	}	
}

/*--------------------------------***  CHECKCANRUN  ***---------------------------------*/
/*	

Examine the machine environment to see if this will run on this Mac. By default, it can.

----------------------------------------------------------------------------------------*/


Boolean		ZApplication::CheckCanRun()
{
	// Returning FALSE will cause the program to
	// immediately show an alert and quit.
	
	return TRUE;
}

/*------------------------------------***  RUN  ***-------------------------------------*/
/*	

Fetch events and handle them until the user quits

----------------------------------------------------------------------------------------*/

void		ZApplication::Run()
{
	// runs the show by asking the event object to get events and handle them. This goes on
	// forever until done is set to TRUE.
	
	if (phase == kRunning)
	{
		while(! done)
		{
			try
			{
				// repeatedly get an event, handle an event
				
				zEH->GetAnEvent();
				zEH->DispatchAnEvent();
				
				// deal with the memory shortage situation, if one has arisen as
				// a result of the last event.
				
				if (memIsShort)
				{
					if (! userHasSeenAlert)
					{
						Alert(kMemoryLowAlertID, NULL);
						userHasSeenAlert = TRUE;
					}
					
					// here, try to replenish the fund if there is enough free memory
					
					if (FreeMem() > (kShortageFundSize * 4))
					{
						memIsShort = FALSE;
						
						Handle temp = NewHandle(kShortageFundSize);
						
						// if that resets memIsShort, then the grow zone func was called, so we
						// are not in the clear yet. However, if the grow zone func wasn't
						// called, then we can safely get rid of that handle and replace it with
						// this one
						
						if (memIsShort)
							DisposeHandle(temp);
						else
						{
							DisposeHandle(shortageFund);
							shortageFund = temp;
							userHasSeenAlert = FALSE;
						}
					}	
				}
			}
			catch(OSErr theErr)
			{
				// if here, an exception was thrown.
				
				Str15	errStr;
				
				NumToString(theErr, errStr);
				ParamText(errStr,NULL,NULL,NULL);
				Alert(kExceptionAlertID,NULL);
				
				if (memIsShort)
					userHasSeenAlert = TRUE;
			}
		}
		done = TRUE;
		phase = kQuitting;
	}
}

/*-----------------------------------***  QUIT  ***-------------------------------------*/
/*	

close all of the windows. If successful, delete the application and return TRUE

----------------------------------------------------------------------------------------*/

Boolean		ZApplication::Quit()
{
	if (phase == kQuitting)
	{
		CloseAll();
		
		if (done)
			ForgetThis();
			
		// g'bye!
	}
	return done;
}


/*-------------------------------***  REQUESTQUIT  ***----------------------------------*/
/*	

ask the application to quit. This is called by choosing Quit from the File menu, for example.

----------------------------------------------------------------------------------------*/


void		ZApplication::RequestQuit()
{
	done = TRUE;
}


/*------------------------------***  HANDLECOMMAND  ***---------------------------------*/
/*	

Handle commands (menu items) that apply to the application as a whole, like Quit.

----------------------------------------------------------------------------------------*/

void		ZApplication::HandleCommand(short menuID, short itemID)
{
	// handle commands at the application level. This includes quit, new, open, etc.
	
	GrafPtr		savePort;
	FSSpec		aFile;
	OSType		anFType;
	Str255		daName;
	
	switch (menuID)
	{
		case kAppleMenuID:
			switch (itemID)
			{
				case kShowAboutBox:
					// put up the "about" box
					Alert(kAboutBoxID, NULL);
					break;
				default:
					// open desk accessories.
					GetItem(GetMHandle(menuID),itemID,daName);
					GetPort(&savePort);
					OpenDeskAcc(daName);
					SetPort(savePort);
					break;
			}
			break;
		case kFileMenu:
			switch (itemID)
			{
				case kItemNew:
					OpenNewWindow();					// open a new "untitled" window
					break;
				case kItemOpen:
					if (PickFile(&aFile,&anFType))		// choose a file
						OpenFile(&aFile, anFType);		// open the file into a window
					break;
				case kItemQuit:
					RequestQuit();						// the app should now quit
					break;
			}
			break;
	}
}

/*-------------------------------***  UPDATEMENUS  ***----------------------------------*/
/*	

enable the menu commands that the app cvan handle at the moment.

----------------------------------------------------------------------------------------*/

void	ZApplication::UpdateMenus()
{
	// enable the menu commands that pertain to the application. This includes "New",
	// "Quit" and "About"
	
	MenuHandle	theMenu;
	
	// apple menu
	
	theMenu = GetMHandle(kAppleMenuID);
	if (theMenu)
		EnableItem(theMenu, kShowAboutBox);
		
	// file menu
	
	theMenu = GetMHandle(kFileMenu);
	if (theMenu)
	{
		if (! memIsShort)
		{
			EnableItem(theMenu, kItemNew);
			EnableItem(theMenu, kItemOpen);
		}
		EnableItem(theMenu, kItemQuit);
	}
}


/*-------------------------------***  INITMENUBAR  ***----------------------------------*/
/*	

load the menubar from a 'MBAR' resource, and also install the Apple Menu items.

----------------------------------------------------------------------------------------*/

void		ZApplication::InitMenuBar()
{
	// installs the menu bar. By default, we just install 'MBAR' ID = 128, which means you
	// don't need to override this to get other menus- just create the resources you want.

	Handle		mBarH;
	MenuHandle	theMenu;
	
	mBarH = GetNewMBar(kStdMenubarID);
	
	if (mBarH)
	{
		SetMenuBar(mBarH);
		ReleaseResource(mBarH);
		
		// get the apple menu and add the DA names
		
		theMenu = GetMHandle(kAppleMenuID);
		
		if (theMenu)
			AddResMenu(theMenu, 'DRVR');
			
		DrawMenuBar();
	}
}

/*------------------------------***  MAKEEVENTHANDLER  ***------------------------------*/
/*	

Make the event handler object. By default, this is a ZEventHandler.

----------------------------------------------------------------------------------------*/


void		ZApplication::MakeEventHandler()
{
	FailNIL(zEH = new ZEventHandler());
	
	zEH->InstallApplescriptHandlers();		// install handlers for the four required events
}

/*------------------------------***  MAKENEWWINDOW  ***---------------------------------*/
/*	

creates a new window object and initialises it. It does not show it yet.

----------------------------------------------------------------------------------------*/

void		ZApplication::MakeNewWindow()
{
	FailNIL(mostRecent = new ZWindow( this,  kUntitledWindowID ));
	
	try
	{
		mostRecent->InitZWindow();
	}
	catch(OSErr err)
	{
		ForgetObject(mostRecent);
		
		throw err;
	}
}


/*------------------------------***  OPENNEWWINDOW  ***---------------------------------*/
/*	

creates a new window object and initialises it. It then shows it and makes it active.

----------------------------------------------------------------------------------------*/

void		ZApplication::OpenNewWindow()
{
	MakeNewWindow();
	mostRecent->Show();
	mostRecent->Select();
}


/*---------------------------------***  CLOSEALL  ***-----------------------------------*/
/*	

closes all of the application's windows. If one refuses to close, this will abort a Quit.

----------------------------------------------------------------------------------------*/

void		ZApplication::CloseAll()
{
	// closes all of the windows. Called as part of the quit operation.

	WindowPtr	w;
	ZWindow*	zappWindow;
	
	w = FrontWindow();
	
	while(w)
	{
		zappWindow = GetZWindow(w);
		
		if (zappWindow)
		{
			if (! zappWindow->Close( phase ))
			{
				phase = kRunning;
				done = FALSE;
				
				break;
			}
		}
		w = FrontWindow();
	}
}

/*------------------------------***  GETFRONTWINDOW  ***--------------------------------*/
/*	

returns the active window object, if there is one

----------------------------------------------------------------------------------------*/

ZWindow*	ZApplication::GetFrontWindow()
{
	WindowPtr	w = FrontWindow();
	
	if (w)
		return GetZWindow(w);
	else
		return NULL;
}


/*----------------------------------***  PICKFILE  ***----------------------------------*/
/*	

displays the standard file dialog for selecting a file, and returns its filespec.

----------------------------------------------------------------------------------------*/


Boolean		ZApplication::PickFile(FSSpec* aFile, OSType* fType)
{
	// uses standard file to choose a file to open to a window
	
	StandardFileReply	aReply;
	
	StandardGetFile(NULL, gNumFileTypes, gFileTypeList, &aReply);
	
	if (aReply.sfGood)
	{
		*aFile = aReply.sfFile;
		*fType = aReply.sfType;
		return TRUE;
	}
	else
		return FALSE;

}


/*----------------------------------***  OPENFILE  ***----------------------------------*/
/*	

creates a new window and asks it to open the file. It then shows the window and activates it.

----------------------------------------------------------------------------------------*/

void		ZApplication::OpenFile(FSSpec* aFile, OSType fType)
{
	// opens the file into a new window
	
	MakeNewWindow();
	
	try
	{
		mostRecent->SetFile(aFile);
		mostRecent->OpenFile( fType );
		mostRecent->Show();
		mostRecent->Select();
	}
	catch(OSErr err)
	{
		ForgetObject(mostRecent);
		throw err;
	}
}





Boolean		ZApplication::MemoryShortage(Size bytesShort)
{
	// this is called when the memory manager gets into dire straits. We can free some or all
	// of our emergency fund to satisfy the request. If we succeed, we return TRUE, else FALSE.
	
	Size	fundSize;
	
	fundSize = GetHandleSize(shortageFund);
	
	if (fundSize <= bytesShort)
		SetHandleSize(shortageFund, 0);
	else
		SetHandleSize(shortageFund, fundSize - bytesShort);
	
	memIsShort = TRUE;
	return (fundSize > bytesShort);
}



static pascal long ZGrowFunc(Size bytesShort)
{
	return gApplication->MemoryShortage(bytesShort);
}


