/*************************************************************************************************
*
*
*			ObjectMacZapp		-- a standard Mac OOP application template
*
*
*
*			ZEventHandler.cpp		-- the event handler object
*
*
*
*
*
*			 1996, Graham Cox
*
*
*
*
*************************************************************************************************/

#include	"ZEventHandler.h"
#include	"ZApplication.h"
#include	"ZWindow.h"
#include	"ZDefines.h"
#include	"ZCommander.h"
#include	"ZErrors.h"
#include	<AppleEvents.h>

// gSleep is the sleep period passed to WaitNextEvent

short			gSleep = kFrontSleep;

// gCurHandler is the current ZCommander object that wants to handle commands. This will
// usually be the active window or the application

ZCommander*		gCurHandler = NULL;

// prototypes of handler functions for requires apple-events

static pascal OSErr			ZOpenAppHandler(AppleEvent* aeEvt, AppleEvent* reply, long refCon);
static pascal OSErr			ZOpenDocsHandler(AppleEvent* aeEvt, AppleEvent* reply, long refCon);
static pascal OSErr			ZPrintDocsHandler(AppleEvent* aeEvt, AppleEvent* reply, long refCon);
static pascal OSErr			ZQuitAppHandler(AppleEvent* aeEvt, AppleEvent* reply, long refCon);


/*-------------------------------***  CONSTRUCTOR  ***---------------------------------*/

ZEventHandler::ZEventHandler()
{
	clicks = 0;
	inBackground = FALSE;
	
	gCurHandler = gApplication;
}


/*---------------------------------***  GETANEVENT  ***---------------------------------*/
/*	

gets the next event off the queue. If it's a null event, give idle time

----------------------------------------------------------------------------------------*/

void	ZEventHandler::GetAnEvent()
{
	// gets an event from the event queue and sets "theEvent" to it.
	
	if (! WaitNextEvent(everyEvent, &theEvent, gSleep, NULL))
		PassIdle();
}


/*-----------------------------***  DISPATCHANEVENT  ***--------------------------------*/
/*	

determine what sort of event it is and call the relevant handler method.

----------------------------------------------------------------------------------------*/

void	ZEventHandler::DispatchAnEvent()
{
	// dispatches the event according to its type, etc
	
	Boolean		isAutoKey = FALSE;
	
	switch (theEvent.what)
	{
		case mouseDown:
			HandleMouseEvent(&theEvent);
			break;
		case activateEvt:
			HandleWindowActivate(	(WindowPtr) theEvent.message,
									(theEvent.modifiers & activeFlag) );
			break;
		case autoKey:
			isAutoKey = TRUE;
		case keyDown:
		{
			char 		theKey;
			long		mSelect;
			ZWindow*	tw;
			
			theKey = theEvent.message & charCodeMask;
			
			if (theEvent.modifiers & cmdKey)
			{
				// if this is a command-key combination, translate this to a
				// menu command as if it were picked from the menu.
				
				if (! isAutoKey)
				{
					mSelect = MenuKey(theKey);		
				
					if (mSelect)
						HandleMenuSelection(mSelect);
				}	
				break;
			}

			// the user is typing- send the characters to the active window
			
			if (tw = gApplication->GetFrontWindow())
				tw->Type(theKey);
			break;
		}
		case updateEvt:
			HandleWindowUpdate((WindowPtr) theEvent.message);
			break;
		case diskEvt:
		{
			Point dPt = {100,100};
			
			if (HiWord(theEvent.message) != 0)
				DIBadMount(dPt, theEvent.message);
			break;
		}
		case kHighLevelEvent:
			HandleHLEvent(&theEvent);
			break;
		case osEvt:
			HandleOSEvent(&theEvent);
			break;
	}
}


/*-------------------------------***  COUNTCLICKS  ***----------------------------------*/
/*	

count up the number of rapid mouse-clicks to detect double-clicks, etc.

----------------------------------------------------------------------------------------*/

void	ZEventHandler::CountClicks(WindowPtr target, long clickTicks)
{
	static WindowPtr 	prevTarget = NULL;
	static long			lastClickTicks = 0;

	if (target == prevTarget)
	{
		if (clickTicks < lastClickTicks + GetDblTime())
			clicks++;
		else
			clicks = 1;
	}
	else
		clicks = 1;
		
	prevTarget = target;
	lastClickTicks = clickTicks;
}


/*---------------------------***  HANDLEMOUSEEVENT  ***---------------------------------*/
/*	

handle mouse button clicks. This finds the target window or other part of the screen

----------------------------------------------------------------------------------------*/

void	ZEventHandler::HandleMouseEvent(EventRecord* theEvent)
{
	WindowPtr		targetWindow;
	short			partCode;
	long			mSelect;
	Point			mousePt;
	ZWindow*		zappWindow;
	
	// find out what part of the screen the mouse was clicked on
	
	partCode = FindWindow(theEvent->where, &targetWindow);
	
	// if it was any sort of window, get the ZWindow object associated with it, if any.
	
	if (targetWindow)
		zappWindow = GetZWindow(targetWindow);
	
	switch (partCode)
	{	
		case inDesk:
			break;
		case inMenuBar:
			// the user clicked the menubar.
			DisableMenus();							// grey out all of the menu items
			
			if (gCurHandler)
				gCurHandler->UpdateMenus();			// ask the current commander to enable its menus, all
													// the way back up to the top of the chain, which is
													// the application.
				
			mSelect = MenuSelect(theEvent->where);	// pull down the menu and track it
			
			if (HiWord(mSelect))
				HandleMenuSelection(mSelect);		// handle the user's choice	
			break;
		case inSysWindow:
			break;
		case inContent:
			if (targetWindow)						// the user clicked in a window
			{
				CountClicks(targetWindow, theEvent->when);	// double click?
				
				if (zappWindow == gApplication->GetFrontWindow() &&
					zappWindow != NULL)
				{
					// if the window is already active, pass the click to the window object
					// in its local coordinate system.
					
					mousePt = theEvent->where;
					
					zappWindow->Focus();
					GlobalToLocal(&mousePt);
					zappWindow->Click(mousePt, theEvent->modifiers);	
				}
				else
				{
					// if the window wasn't active, make it active.
					
					if (zappWindow)
						zappWindow->Select();
				}
			}
			break;
		case inDrag:
		{
			// the user wants to drag the window
			
			Rect limitRect;
			
			limitRect = (*GetGrayRgn())->rgnBBox;
			InsetRect(&limitRect,4,4);
			
			DragWindow(targetWindow, theEvent->where, &limitRect);
			
			break;
		}
		case inGrow:
		{
			// the user wants to resize the window
			
			Rect szRect;
			
			if (zappWindow)
				zappWindow->GetSizeRect(&szRect);
				
			mSelect = GrowWindow(targetWindow, theEvent->where, &szRect);
			
			if (mSelect)
			{
				if (zappWindow)
					zappWindow->SetSize(LoWord(mSelect), HiWord(mSelect));
			}
			break;
		}
		case inGoAway:
			if (TrackGoAway(targetWindow, theEvent->where))
			{
				// if the user clicks in the go-away box, the window will be closed
				
				if (zappWindow)
					zappWindow->Close(gApplication->GetPhase());
			}
			break;
		case inZoomIn:
		case inZoomOut:
			if (TrackBox(targetWindow, theEvent->where, partCode))
			{
				// if the user clicked in the zoom box, the window will be zoomed
				
				if (zappWindow)
					zappWindow->Zoom(partCode);
			}
			break;
	}
}


/*---------------------------***  HANDLEWINDOWUPDATE  ***-------------------------------*/
/*	

to handle an update event, focus the window and ask it do draw its contents.

----------------------------------------------------------------------------------------*/

void	ZEventHandler::HandleWindowUpdate(WindowPtr theWindow)
{
	ZWindow*		zappWindow = NULL;
	GrafPtr			savePort;
	Boolean			memState;
	
	if (theWindow)
		zappWindow = GetZWindow(theWindow);
	
	if (zappWindow)
	{
		memState = gApplication->memIsShort;
		
		GetPort(&savePort);
		zappWindow->Focus();
		
		BeginUpdate(theWindow);
		if (! memState)
			zappWindow->Draw();
		EndUpdate(theWindow);
		
		// if the draw resulted in a memory shortage, suppress the warning, since
		// that will trigger further updates, etc.
		
		if (gApplication->memIsShort != memState)
			gApplication->userHasSeenAlert = TRUE;
			
		SetPort(savePort);	
	}
}



/*---------------------------***  HANDLEMENUSELECTION  ***------------------------------*/
/*	

send the chosen menu command to the current handler, which will pass unhandled commands
back up the chain until they arrive at the application.

----------------------------------------------------------------------------------------*/


void	ZEventHandler::HandleMenuSelection(long mSelect)
{
	if (gCurHandler)
		gCurHandler->HandleCommand(HiWord(mSelect), LoWord(mSelect));
		
	HiliteMenu(0);
}


/*------------------------------***  DISABLEMENUS  ***----------------------------------*/
/*	

make all menu items grey, except titles and apple menu items

----------------------------------------------------------------------------------------*/


void	ZEventHandler::DisableMenus()
{
	// greys out all menus except apple menu items. Each window then enables menus that
	// pertain to it, then the app does the same.
	
	MenuHandle	theMenu;
	
	theMenu = GetMHandle(kAppleMenuID);
	if (theMenu)
		(*theMenu)->enableFlags = 0xFFFFFFFD;
		
	theMenu = GetMHandle(kFileMenu);
	if (theMenu)
		(*theMenu)->enableFlags = 1;
	
	theMenu = GetMHandle(kEditMenu);
	if (theMenu)
		(*theMenu)->enableFlags = 1;
}


/*---------------------------------***  PASSIDLE  ***-----------------------------------*/
/*	

send the idle command to the current handler, and pass it up the chain

----------------------------------------------------------------------------------------*/


void	ZEventHandler::PassIdle()
{
	// called repeatedly for null events. This passes idle messages to the current commander
	
	if (gCurHandler)
		gCurHandler->Idle();
}



/*--------------------------***  HANDLEWINDOWACTIVATE  ***------------------------------*/
/*	

calls activate/deactivate for window, and sets up the current handler

----------------------------------------------------------------------------------------*/

void	ZEventHandler::HandleWindowActivate(WindowPtr theWindow, Boolean state)
{
	// handles the activate and deactivate events. This calls the window object with
	// the activate messages, and also sets up gCurHandler.

	ZWindow*		zappWindow = NULL;
	
	if (theWindow)
		zappWindow = GetZWindow(theWindow);
	
	if (zappWindow)
	{
		if (state)
		{
			gCurHandler = zappWindow->GetHandler();
			zappWindow->Focus();
			zappWindow->Activate();
		}
		else
		{
			zappWindow->Focus();
			zappWindow->Deactivate();
			gCurHandler = zappWindow->GetBoss();
		}
	}
}


/*------------------------------***  HANDLEOSEVENT  ***---------------------------------*/
/*	

handle the suspend and resume events sent by MultiFinder/System 7

----------------------------------------------------------------------------------------*/

void	ZEventHandler::HandleOSEvent(EventRecord* theEvent)
{
	if (theEvent->message & suspendResumeMessage)
	{
		inBackground = FALSE;
		gSleep = kFrontSleep;	
	}
	else
	{
		inBackground = TRUE;
		gSleep = kBackSleep;
	}
}


/*------------------------------***  HANDLEHLEVENT  ***---------------------------------*/
/*	

handle the high level events sent by AOCE, Applescript, et al.

----------------------------------------------------------------------------------------*/

void	ZEventHandler::HandleHLEvent(EventRecord* theEvent)
{
	// handles high level events such as applescript events
	
	(void) AEProcessAppleEvent(theEvent);
}


/*-----------------------***  INSTALLAPPLESCRIPTHANDLERS  ***---------------------------*/
/*	

install the handler vectors needed to field applescript events.

----------------------------------------------------------------------------------------*/

void	ZEventHandler::InstallApplescriptHandlers()
{
	// installs the default handlers for the required events
	
	OSErr				theErr;
	AEEventHandlerUPP	aeUPP;
	
	aeUPP = NewAEEventHandlerProc(ZOpenAppHandler);
	theErr = AEInstallEventHandler(kCoreEventClass, kAEOpenApplication, aeUPP, (long) this, FALSE);
	aeUPP = NewAEEventHandlerProc(ZOpenDocsHandler);
	theErr = AEInstallEventHandler(kCoreEventClass, kAEOpenDocuments,	aeUPP, (long) this, FALSE);
	aeUPP = NewAEEventHandlerProc(ZPrintDocsHandler);
	theErr = AEInstallEventHandler(kCoreEventClass, kAEPrintDocuments, 	aeUPP, (long) this, FALSE);
	aeUPP = NewAEEventHandlerProc(ZQuitAppHandler);
	theErr = AEInstallEventHandler(kCoreEventClass, kAEQuitApplication, aeUPP, (long) this, FALSE);
}



static pascal OSErr			ZOpenAppHandler(AppleEvent* aeEvt, AppleEvent* reply, long refCon)
{
	gApplication->OpenNewWindow();
	
	return noErr;
}


static pascal OSErr			ZOpenDocsHandler(AppleEvent* aeEvt, AppleEvent* reply, long refCon)
{
	FSSpec		aFile;
	OSErr		theErr = noErr;
	AEDescList	docList;
	long		i,n;
	AEKeyword	keyWrd;
	DescType	retType;
	Size		actualSize;
	FInfo		fi;
	
	try
	{
		FailOSErr(AEGetParamDesc( aeEvt, keyDirectObject, typeAEList, &docList));	
		FailOSErr(AECountItems(&docList, &n));
		
		for (i = 1; i <= n; i++)
		{
			FailOSErr(AEGetNthPtr( &docList,
									i, 
									typeFSS, 
									&keyWrd,
									&retType, 
									&aFile, 
									sizeof(FSSpec), 
									&actualSize));
			
			FailOSErr(FSpGetFInfo(&aFile, &fi));
			
			gApplication->OpenFile(&aFile, fi.fdType);
		}
		
		FailOSErr(AEDisposeDesc(&docList));
	}
	catch(OSErr err)
	{
		theErr = err;
	}
	return theErr;
}


static pascal OSErr			ZPrintDocsHandler(AppleEvent* aeEvt, AppleEvent* reply, long refCon)
{
	return errAEEventNotHandled;
}


static pascal OSErr			ZQuitAppHandler(AppleEvent* aeEvt, AppleEvent* reply, long refCon)
{
	gApplication->RequestQuit();
	
	return noErr;
}
