/*************************************************************************************************
*
*
*			ObjectMacZapp		-- a standard Mac OOP application template
*
*
*
*			ZWindow.cpp			-- the window object
*
*
*
*
*
*			 1996, Graham Cox
*
*
*
*
*************************************************************************************************/


#include	"ZWindow.h"
#include	"ZApplication.h"
#include	"ZEventHandler.h"
#include	"ZDefines.h"
#include	"ZErrors.h"


/*------------------------------***  CONSTRUCTOR  ***---------------------------------*/

ZWindow::ZWindow(ZCommander* aBoss, short windowID)
	: ZCommander(aBoss)
{
	windID = windowID;
	macWindow = NULL;
	dirty = FALSE;
	isNamed = FALSE;
	macFile.vRefNum = kNoFile;
	
	SetRect(&sizeRect,50,50,2000,2000);
}

/*------------------------------***  DESTRUCTOR  ***---------------------------------*/

ZWindow::~ZWindow()
{
	if (macWindow)
		DisposeWindow(macWindow);
}


/*--------------------------------***  INITZWINDOW  ***---------------------------------*/
/*	

initialise this object. By default, this just makes the window

----------------------------------------------------------------------------------------*/

void	ZWindow::InitZWindow()
{
	MakeMacWindow(windID);
}


/*------------------------------***  MAKEMACWINDOW  ***---------------------------------*/
/*	

create the macintosh window that this object looks after

----------------------------------------------------------------------------------------*/

void	ZWindow::MakeMacWindow(short windID)
{
	if (gIsAColourMac)
		FailNIL(macWindow = GetNewCWindow(windID, NULL, NULL));
	else
		FailNIL(macWindow = GetNewWindow(windID, NULL, NULL));
	
	((WindowPeek) macWindow)->windowKind = IS_ZWINDOW_KIND;
	SetWRefCon(macWindow, (long) this);
}


/*----------------------------------***  ACTIVATE  ***----------------------------------*/
/*	

this window is becoming active

----------------------------------------------------------------------------------------*/

void	ZWindow::Activate()
{
	// the window is activated
	
	DrawGrow();
}


/*---------------------------------***  DEACTIVATE  ***---------------------------------*/
/*	

this window is becoming inactive

----------------------------------------------------------------------------------------*/

void	ZWindow::Deactivate()
{
	// the window is no longer active

	DrawGrow();
}


/*------------------------------------***  FOCUS  ***-----------------------------------*/
/*	

make this window the current port

----------------------------------------------------------------------------------------*/

void	ZWindow::Focus()
{
	SetPort(macWindow);
	SetOrigin(0,0);
}


/*------------------------------------***  DRAW  ***------------------------------------*/
/*	

draw the contents of this window. This is the draw dispatcher.

----------------------------------------------------------------------------------------*/

void	ZWindow::Draw()
{
	DrawContent();
	DrawGrow();	
}


/*-------------------------------***  DRAWCONTENT  ***----------------------------------*/
/*	

draw the content area of this window- this is what will draw the meaningful part

----------------------------------------------------------------------------------------*/

void	ZWindow::DrawContent()
{
	EraseRect(&macWindow->portRect);
}


/*------------------------------***  GETCONTENTRECT  ***--------------------------------*/
/*	

return the content area

----------------------------------------------------------------------------------------*/

void	ZWindow::GetContentRect(Rect* contents)
{
	*contents = macWindow->portRect;
}



/*---------------------------------***  DRAWGROW  ***-----------------------------------*/
/*	

this window is becoming active

----------------------------------------------------------------------------------------*/

void	ZWindow::DrawGrow()
{
	short	v = GetWVariant(macWindow);
	
	if (v == documentProc || v == zoomDocProc)
		DrawGrowIcon(macWindow);
}


/*-----------------------------------***  CLICK  ***------------------------------------*/
/*	

the user clicked in the content area of this window

----------------------------------------------------------------------------------------*/

void	ZWindow::Click(Point mouse, short modifiers)
{
}


/*-----------------------------------***  TYPE  ***-------------------------------------*/
/*	

this user is typing while this window is active

----------------------------------------------------------------------------------------*/

void	ZWindow::Type(char theChar)
{
}


/*-------------------------------***  SETSIZERECT  ***----------------------------------*/
/*	

set minimum and maximum sizes for this window

----------------------------------------------------------------------------------------*/

void	ZWindow::SetSizeRect(Rect* szRect)
{
	sizeRect = *szRect;
}


/*-------------------------------***  GETSIZERECT  ***----------------------------------*/
/*	

get minimum and maximum sizes for this window

----------------------------------------------------------------------------------------*/

void	ZWindow::GetSizeRect(Rect* szRect)
{
	*szRect = sizeRect;
}


/*----------------------------------***  CLOSE  ***-------------------------------------*/
/*	

we want to close this window. If it needs to be saved, do that. The user may cancel this.

----------------------------------------------------------------------------------------*/

Boolean	ZWindow::Close(short phase)
{
	// the user wants to close the window. This checks the dirty flag and if dirty, asks the
	// user if they want to save changes, if so, this calls save.
	
	short check = kCloseNoSave;
	
	if (dirty)
	{
		Str31	nameStr;
		Str31	phaseStr;
		
		GetName(nameStr);
		GetIndString(phaseStr, kMiscStrListID, (phase == kRunning)? 1 : 2);
		ParamText(nameStr,phaseStr,NULL,NULL);
		
		check = Alert( kConfirmSaveAlertID, NULL);		// do you wish to save?
		
		if (check == kConfirmSave)
			if (! Save( FALSE ))						// if so, save the file
				check = kCloseCancel;
	}
	
	if (check != kCloseCancel)
	{
		Hide();
		
		if (gCurHandler == this)						// make the current handler this
			gCurHandler = itsBoss;						// one's boss- i.e. the application
		
		ForgetThis();
		return TRUE;	// was closed
	}
	else
		return FALSE;	// wasn't closed
}


/*-----------------------------------***  HIDE  ***-------------------------------------*/
/*	

make this window invisible

----------------------------------------------------------------------------------------*/

void	ZWindow::Hide()
{
	HideWindow(macWindow);
}


/*-----------------------------------***  SHOW  ***-------------------------------------*/
/*	

make this window visible

----------------------------------------------------------------------------------------*/

void	ZWindow::Show()
{
	ShowWindow(macWindow);
}


/*---------------------------------***  SELECT  ***-------------------------------------*/
/*	

make this window the active window

----------------------------------------------------------------------------------------*/

void	ZWindow::Select()
{
	SelectWindow(macWindow);
}


/*---------------------------------***  PLACEAT  ***------------------------------------*/
/*	

position the window on the screen at h, v.

----------------------------------------------------------------------------------------*/

void	ZWindow::PlaceAt(short hGlobal, short vGlobal)
{
	MoveWindow(macWindow, hGlobal, vGlobal, FALSE);
}


/*------------------------------***  HANDLECOMMAND  ***---------------------------------*/
/*	

handle commands for a window, like Save, Save As, Close, etc.

----------------------------------------------------------------------------------------*/

void	ZWindow::HandleCommand(short menuID, short itemID)
{
	
	if (menuID == kFileMenu)
	{
		switch (itemID)
		{
			case kItemClose:
				Close( kRunning );
				break;
			case kItemSave:
				Save( FALSE );
				break;
			case kItemSaveAs:
				Save( TRUE );
				break;
			case kItemRevert:
				Revert();
				break;
		}
	}
	// pass other commands up to this object's boss (the app, in this case)
	
	inherited::HandleCommand(menuID, itemID);
}


/*--------------------------------***  UPDATEMENUS  ***---------------------------------*/
/*	

updet the menus for a window, like Save, Save As, Close, etc.

----------------------------------------------------------------------------------------*/

void	ZWindow::UpdateMenus()
{
	// this enables the Close, Save and SaveAs commands.
	
	MenuHandle	theMenu;
	
	theMenu = GetMHandle(kFileMenu);
	
	if (theMenu)
	{
		EnableItem( theMenu, kItemClose);
		
		if (dirty)
		{
			EnableItem( theMenu, kItemSave);
			
			if (isNamed)
				EnableItem( theMenu, kItemRevert);
		}
		EnableItem( theMenu, kItemSaveAs);
	}
	
	// call the boss to enable his menus
	
	inherited::UpdateMenus();
}


/*---------------------------------***  SETSIZE  ***------------------------------------*/
/*	

set this window to the width and height passed

----------------------------------------------------------------------------------------*/

void	ZWindow::SetSize(short width, short height)
{
	// sets the windows size to the width and height passed, constrained to sizerect.
	
	Focus();
	EraseRect(&macWindow->portRect);
	SizeWindow(macWindow, width, height, TRUE);
	Draw();
	DrawGrow();
}


/*-----------------------------------***  ZOOM  ***-------------------------------------*/
/*	

zoom this window betwen the standard and user states

----------------------------------------------------------------------------------------*/

void	ZWindow::Zoom(short partCode)
{
	Focus();
	EraseRect(&macWindow->portRect);
	ZoomWindow(macWindow, partCode, FALSE);
}


/*-----------------------------------***  SAVE  ***-------------------------------------*/
/*	

save the contents of the window to a file. This may display the standard file dialog.

----------------------------------------------------------------------------------------*/


Boolean	ZWindow::Save(Boolean forceSaveAs)
{
	// handles the save and save as commands.
	
	if (! isNamed || forceSaveAs)
	{
		// do Save As
		
		StandardFileReply	macReply;
		Str31				prompt,name;
		
		GetIndString( prompt, kMiscStrListID, 3 );
		GetName( name );
		StandardPutFile( prompt, name, &macReply);
		
		if (macReply.sfGood)
			macFile = macReply.sfFile;
		else
			return FALSE;		// user cancelled the save
	}
	
	// now do a save with the info.
	
	SaveFile();
	
	return TRUE;
}


/*---------------------------------***  SAVEFILE  ***-----------------------------------*/
/*	

write out the contents of the window to a file. This may rename the window.

----------------------------------------------------------------------------------------*/


void	ZWindow::SaveFile()
{
	// actually save the window's contents to the file.
	
	if (macFile.vRefNum != kNoFile)
	{
		SetWTitle(macWindow, macFile.name);
		
		// no longer dirty
		
		isNamed = TRUE;
		dirty = FALSE;
	}
}


/*---------------------------------***  GETNAME  ***------------------------------------*/
/*	

return the name of the window

----------------------------------------------------------------------------------------*/

void	ZWindow::GetName(Str255 name)
{
	GetWTitle(macWindow, name);
}



/*----------------------------------***  REVERT  ***------------------------------------*/
/*	

revert the contents of the window to the original file

----------------------------------------------------------------------------------------*/

void	ZWindow::Revert()
{
	// revert the contents. By default, this just calls open.
	
	OpenFile( 0 );
}


/*---------------------------------***  SETFILE  ***------------------------------------*/
/*	

set the filespec for this window to the file passed.

----------------------------------------------------------------------------------------*/

void	ZWindow::SetFile(FSSpec* aFile)
{
	macFile = *aFile;
}


/*---------------------------------***  OPENFILE  ***-----------------------------------*/
/*	

open the file into the window's contents. This will set the name of the window to equal
the filename

----------------------------------------------------------------------------------------*/

void	ZWindow::OpenFile(OSType aFileType)
{
	// this opens the current file into this window. This function should read the
	// contents of the file, replacing the current contents. The default method
	// does nothing. You should check that the vRefNum of the file spec is no kNoFile.
	
	if (macFile.vRefNum != kNoFile)
	{
		SetWTitle(macWindow, macFile.name);
		isNamed = TRUE;
		dirty = FALSE;
	}
}


/*--------------------------------***  GETHANDLER  ***----------------------------------*/
/*	

return the object that begins the chain of command for this window. By default, it is the
window itself.

----------------------------------------------------------------------------------------*/

ZCommander*	ZWindow::GetHandler()
{
	return this;
}


